# -*- coding: utf-8 -*-
import sys
from importlib import reload

from aliyunsdkdysmsapi.request.v20170525 import SendSmsRequest
from aliyunsdkcore.client import AcsClient
import uuid
from aliyunsdkcore.profile import region_provider
import random

try:
    reload(sys)
except NameError:
    pass
except Exception as err:
    raise err

# 注意：不要更改
REGION = "cn-hangzhou"
PRODUCT_NAME = "Dysmsapi"
DOMAIN = "dysmsapi.aliyuncs.com"

acs_client = AcsClient('LTAI4Fe85gjtzMJfvCXrbr6j', 'M1U2QLUhnE6KiQGpK16n81ZlzA8uQr', REGION)
region_provider.add_endpoint(PRODUCT_NAME, REGION, DOMAIN)


def send_sms(business_id, phone_numbers, sign_name, template_code, template_param=None):
    smsRequest = SendSmsRequest.SendSmsRequest()
    # 申请的短信模板编码,必填
    smsRequest.set_TemplateCode(template_code)

    # 短信模板变量参数
    if template_param is not None:
        smsRequest.set_TemplateParam(template_param)

    # 设置业务请求流水号，必填。
    smsRequest.set_OutId(business_id)

    # 短信签名
    smsRequest.set_SignName(sign_name)

    # 数据提交方式
    # smsRequest.set_method(MT.POST)

    # 数据提交格式
    # smsRequest.set_accept_format(FT.JSON)

    # 短信发送的号码列表，必填。
    smsRequest.set_PhoneNumbers(phone_numbers)

    # 调用短信发送接口，返回json
    smsResponse = acs_client.do_action_with_exception(smsRequest)
    print(smsResponse)

    return smsResponse


if __name__ == '__main__':
    __business_id = uuid.uuid1()
    # print(__business_id)
    num = random.randrange(100000, 999999)
    params = "{'code':123456}"
    params = params.replace("123456", str(num))
    # params = 123456
    # params = u'{"name":"wqb","code":"12345678","address":"bz","phone":"13000000000"}'
    print(send_sms(__business_id, "15365128366", "Treelab企兰", "SMS_204761722", params))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2018年2月5日13:37:54
Desc    ：记录API
"""


class KerriganAPIS:
    kerrigan_prefix = "/api/kerrigan"
    get_publish_config = dict(method='GET',
                              url=f'{kerrigan_prefix}/v1/conf/publish/config/',
                              params={'project_code': '',
                                      'environment': '',
                                      'service': 'service',
                                      'filename': 'filename'},
                              field_help={},
                              description='获取配置'
                              )



"""
处理API请求
"""
from abc import ABC

from shortuuid import uuid
import traceback
from .cache_context import cache_conn
from tornado.web import RequestHandler, HTTPError
from .jwt_token import AuthToken, jwt


class BaseHandler(RequestHandler, ABC):
    def __init__(self, *args, **kwargs):
        self.new_csrf_key = str(uuid())
        self.business_id, self.resource_group = None, None
        self.user_id, self.username, self.nickname, self.email, self.is_super = None, None, None, None, False
        self.is_superuser = self.is_super
        self.token_verify = False
        self.params = {}
        super(BaseHandler, self).__init__(*args, **kwargs)

    def get_params_dict(self):
        self.params = {k: self.get_argument(k) for k in self.request.arguments}
        if "filter_map" in self.params:
            try:
                import json
                filter_map = self.params.get('filter_map')
                filter_map = json.loads(filter_map)
            except:
                filter_map = {}
        else:
            filter_map = {}
        self.params['filter_map'] = filter_map

        if self.request_resource_map and isinstance(self.request_resource_map, dict):
            self.params['filter_map'] = {**filter_map, **self.request_resource_map}
        if "auth_key" in self.params:
            self.params.pop('auth_key')

    def opex_csrf(self):
        # 验证客户端CSRF，如请求为GET，则不验证，否则验证。最后将写入新的key
        cache = cache_conn()
        if self.request.method in ("GET", "HEAD", "OPTIONS") or self.request.headers.get('Sdk-Method'):
            pass
        else:
            csrf_key = self.get_cookie('csrf_key')
            if not csrf_key:
                raise HTTPError(402, 'csrf error need csrf key')
            result = cache.get(csrf_key)
            cache.delete(csrf_key)
            if isinstance(result, bytes):
             result = result.decode()
            if result != '1':
                raise HTTPError(402, 'csrf error')
            cache.set(self.new_csrf_key, '1', ex=24*60*60)
            self.set_cookie('csrf_key', self.new_csrf_key)

    def opex_login(self):
        # 登陆验证
        auth_key = self.get_cookie('auth_key') if self.get_cookie("auth_key") else self.request.headers.get('auth-key')
        if not auth_key:
            auth_key = self.get_argument('auth_key', default=None, strip=True)

        if not auth_key:
            raise HTTPError(401, 'auth failed')

        if self.token_verify:
            auth_token = AuthToken()
            user_info = auth_token.decode_auth_token(auth_key)
        else:
            user_info = jwt.decode(auth_key, options={"verify_signature": False}).get('data')

        if not user_info:
            raise HTTPError(401, 'auth failed')

        self.user_id = user_info.get('user_id', None)
        self.username = user_info.get('username', None)
        self.nickname = user_info.get('nickname', None)
        self.email = user_info.get('email', None)
        self.is_super = user_info.get('is_superuser', False)

        if not self.user_id: raise HTTPError(401, 'auth failed')

        self.user_id = str(self.user_id)
        self.set_secure_cookie("user_id", self.user_id)
        self.set_secure_cookie("nickname", self.nickname)
        self.set_secure_cookie("username", self.username)
        self.set_secure_cookie("email", str(self.email))
        self.is_superuser = self.is_super

    def initialize(self, *args, **kwargs):
        pass

    def prepare(self):
        # 获取url参数为字典
        self.get_params_dict()
        # 验证客户端CSRF
        self.opex_csrf()

        # 登陆验证
        self.opex_login()

    def get_current_user(self):
        return self.username

    def get_current_id(self):
        return self.user_id

    def get_current_nickname(self):
        return self.nickname

    def get_current_email(self):
        return self.email

    def is_superuser(self):
        return self.is_superuser

    @property
    def request_resource_group(self):
        if not self.resource_group:
            self.resource_group = self.get_secure_cookie("resource_group") if self.get_secure_cookie(
                "resource_group") else self.request.headers.get('resource-group')

            if not self.resource_group: return None
            if isinstance(self.resource_group, bytes):  self.resource_group = bytes.decode(self.resource_group)
            return self.resource_group

        return self.resource_group

    @property
    def request_resource_map(self):
        if self.request_resource_group in [None, 'all', '所有项目']:
            return dict()
        else:
            return dict(resource_group=self.request_resource_group)

    @property
    def request_business_id(self):
        if not self.business_id:
            self.business_id = self.get_secure_cookie("business_id") if self.get_secure_cookie("business_id") else \
                self.request.headers.get('biz-id')
            if not self.business_id: return None
            if isinstance(self.business_id, bytes):  self.business_id = bytes.decode(self.business_id)
            return self.business_id

        return self.business_id

    @property
    def request_username(self):
        return self.username

    @property
    def request_user_id(self):
        return self.user_id

    @property
    def request_nickname(self):
        return self.nickname

    @property
    def request_email(self):
        return self.email

    @property
    def request_is_superuser(self):
        return self.is_superuser

    def write_error(self, status_code, **kwargs):
        error_trace_list = traceback.format_exception(*kwargs.get("exc_info"))
        if status_code == 404:
            self.set_status(status_code)
            return self.finish('找不到相关路径-404')

        elif status_code == 400:
            self.set_status(status_code)
            return self.finish('bad request...')

        elif status_code == 402:
            self.set_status(status_code)
            return self.finish('csrf error...')

        elif status_code == 403:
            self.set_status(status_code)
            return self.finish('Sorry, you have no permission. Please contact the administrator')

        if status_code == 500:
            self.set_status(status_code)
            for line in error_trace_list:
                self.write(str(line))
            self.finish()

        elif status_code == 401:
            self.set_status(status_code)
            return self.finish('你没有登录')

        else:
            self.set_status(status_code)


class LivenessProbe(RequestHandler, ABC):
    def initialize(self, *args, **kwargs):
        pass

    def head(self):
        self.write(dict(code=0, msg="I'm OK"))

    def get(self):
        self.write(dict(code=0, msg="I'm OK"))



#!/usr/bin/env python
# -*-coding:utf-8-*-
"""
Author : ss
date   : 2018年4月11日
role   : 缓存
"""

import base64
import json

import redis
from shortuuid import uuid

from .configs import configs as my_configs
from .consts import const
from .tools import singleton, bytes_to_unicode, convert


@singleton
class Cache:
    def __init__(self):
        self.__redis_connections = {}
        redis_configs = my_configs[const.REDIS_CONFIG_ITEM]
        for config_key, redis_config in redis_configs.items():
            auth = redis_config[const.RD_AUTH_KEY]
            host = redis_config[const.RD_HOST_KEY]
            port = redis_config[const.RD_PORT_KEY]
            db = redis_config[const.RD_DB_KEY]
            return_utf8 = False
            if const.RD_DECODE_RESPONSES in redis_config:
                return_utf8 = redis_config[const.RD_DECODE_RESPONSES]
            password = redis_config[const.RD_PASSWORD_KEY]

            if auth:
                redis_conn = redis.Redis(host=host, port=port, db=db, password=password, decode_responses=return_utf8)
            else:
                redis_conn = redis.Redis(host=host, port=port, db=db, decode_responses=return_utf8)
            self.__redis_connections[config_key] = redis_conn

        self.__salt = str(uuid())

    def set(self, key, value, expire=-1, conn_key=const.DEFAULT_RD_KEY, private=True, pipeline=None):
        real_key = self.__get_key(key, private)
        execute_main = self.__get_execute_main(conn_key, pipeline)
        if expire > 0:
            execute_main.set(real_key, value, ex=expire)
        else:
            execute_main.set(real_key, value)

    def set_json(self, key, value, expire=-1, conn_key=const.DEFAULT_RD_KEY, private=True, pipeline=None):
        value = json.dumps(value)
        value = base64.b64encode(value.encode('utf-8'))
        self.set(key, value, expire, conn_key, private, pipeline)

    def get(self, key, default='', conn_key=const.DEFAULT_RD_KEY, private=True, pipeline=None):
        real_key = self.__get_key(key, private)
        execute_main = self.__get_execute_main(conn_key, pipeline)
        if execute_main.exists(real_key):
            result = execute_main.get(real_key)
            return bytes_to_unicode(result)
        return default

    def incr(self, key, private=True,
             conn_key=const.DEFAULT_RD_KEY, amount=1):
        real_key = self.__get_key(key, private)
        execute_main = self.__get_execute_main(conn_key, None)
        if execute_main.exists(real_key):
            execute_main.incr(real_key, amount=amount)
            return self.get(key, default='0',
                            private=private, conn_key=conn_key)
        return None

    def get_json(self, key, default='',
                 conn_key=const.DEFAULT_RD_KEY, private=True):
        result = self.get(key, default, conn_key, private)
        result = base64.b64decode(result)
        result = bytes_to_unicode(result)
        if result:
            result = json.loads(result)
        return result

    def delete(self, *keys, conn_key=const.DEFAULT_RD_KEY, private=True, pipeline=None):
        execute_main = self.__get_execute_main(conn_key, pipeline)
        _keys = [self.__get_key(key, private) for key in keys]
        return execute_main.delete(*_keys)

    def clear(self, conn_key=const.DEFAULT_RD_KEY):
        execute_main = self.__get_execute_main(conn_key, None)
        execute_main.flushdb()

    def get_pipeline(self, conn_key=const.DEFAULT_RD_KEY):
        return self.__redis_connections[conn_key].pipeline()

    def execute_pipeline(self, pipeline):
        if pipeline:
            return pipeline.execute()

    def get_conn(self, conn_key=const.DEFAULT_RD_KEY):
        return self.__get_execute_main(conn_key)

    def hgetall(self, key, default='', conn_key=const.DEFAULT_RD_KEY, private=True):
        real_key = self.__get_key(key, private)
        execute_main = self.__get_execute_main(conn_key, None)
        if execute_main.exists(real_key):
            result = execute_main.hgetall(real_key)
            result = convert(result)
        else:
            return default
        return result

    @property
    def redis(self):
        return self.__get_execute_main()

    def __get_key(self, key, private=True):
        if private:
            return '%s%s' % (self.__salt, key)
        else:
            return key

    def __get_execute_main(self, conn_key=const.DEFAULT_RD_KEY, pipeline=None):
        if pipeline:
            return pipeline
        return self.__redis_connections[conn_key]


def get_cache():
    return Cache()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date   : 2019年2月5日13:37:54
Desc   : 云厂商的一些方法
"""

import requests
import hashlib
import logging

logging.basicConfig(level=logging.INFO, format='%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s',
                    datefmt='%Y %H:%M:%S')

logger = logging.getLogger("ucloud")
logger.setLevel(logging.WARN)


class UCloudApi:
    """UCloud接口"""

    def __init__(self, access_id, access_key, url='https://api.ucloud.cn/'):
        self.url = url
        self.access_id = access_id
        self.access_key = access_key

    def get_region_list(self):
        params = {
            'Action': 'GetRegion',
            'PublicKey': self.access_id
        }
        params = self.add_signature(params)
        req = requests.get(url=self.url, params=params)
        regions = list(set([r['Region'] for r in req.json()['Regions']]))
        return regions

    def get_project_list(self):
        params = {
            'Action': 'GetProjectList',
            'PublicKey': self.access_id
        }
        params = self.add_signature(params)
        req = requests.get(url=self.url, params=params)
        project = list(set([r['ProjectId'] for r in req.json()['ProjectSet']]))
        return project

    def get_project_info(self):
        params = {
            'Action': 'GetProjectList',
            'PublicKey': self.access_id
        }
        params = self.add_signature(params)
        req = requests.get(url=self.url, params=params)
        project_list = []
        for i in req.json()['ProjectSet']:
            if isinstance(i, dict):
                project_list.append(i)
        return project_list

    def add_signature(self, params):
        items = params.items()
        params_data = ""
        for key in sorted(items):
            params_data = params_data + str(key[0]) + str(key[1])
        params_data = params_data + self.access_key
        sign = hashlib.sha1()
        sign.update(params_data.encode('utf8'))
        signature = sign.hexdigest()
        params['Signature'] = signature
        return params



"""
云厂商的一些方法
"""
from .cloud.qcloud_api import QCloudApiOpt
from .cloud.ucloud_api import UCloudApi


def cloud_factory(cloud):
    if cloud == 'aliyun':
        return None

    elif cloud == 'qcloud':
        return QCloudApiOpt()

    elif cloud == 'ucloud':
        return UCloudApi()

    elif cloud == 'aws':
        return None
    else:
        return None



"""
web  log
"""

import logging
import os
import time
from shortuuid import uuid

log_fmt = ''.join(('PROGRESS:%(progress_id) -5s %(levelname) ', '-10s %(asctime)s %(name) -25s %(funcName) -30s LINE.NO:%(lineno) -5d : %(message)s'))

log_key = 'logger_key'


def singleton(class_):
    instances = {}

    def getinstance(*args, **kwargs):
        if class_ not in instances:
            instances[class_] = class_(*args, **kwargs)
        return instances[class_]

    return getinstance


class ProgressLogFilter(logging.Filter):
    def filter(self, record):
        record.progress_id = Logger().progress_id
        return True


@singleton
class Logger:
    def __init__(self, progress_id='', log_file='/tmp/xxx.log'):
        self.__log_key = log_key
        self.progress_id = progress_id
        self.log_file = log_file

    def read_log(self, log_level, log_message):
        # 创建一个logger
        if self.progress_id == '':
            Logger().progress_id = str(uuid())
        else:
            Logger().progress_id = self.progress_id
        logger = logging.getLogger(self.__log_key)
        logger.addFilter(ProgressLogFilter())
        logger.setLevel(logging.DEBUG)

        # 创建一个handler用于输出到终端
        th = logging.StreamHandler()
        th.setLevel(logging.DEBUG)

        # 定义handler的输出格式
        formatter = logging.Formatter(log_fmt)
        th.setFormatter(formatter)

        # 给logger添加handler
        logger.addHandler(th)

        # 记录日志
        level_dic = {'debug': logger.debug, 'info': logger.info, 'warning': logger.warning, 'error': logger.error,
                     'critical': logger.critical}
        level_dic[log_level](log_message)

        th.flush()
        logger.removeHandler(th)

    def write_log(self, log_level, log_message):
        # 创建一个logger
        if self.progress_id == '':
            Logger().progress_id = str(uuid())
        else:
            Logger().progress_id = self.progress_id
        logger = logging.getLogger(self.__log_key)
        logger.addFilter(ProgressLogFilter())
        logger.setLevel(logging.DEBUG)

        # 建立日志目录
        log_dir = os.path.dirname(self.log_file)
        if not os.path.isdir(log_dir):
            os.makedirs(log_dir)

        # 创建一个handler用于写入日志文件
        fh = logging.FileHandler(self.log_file)
        fh.setLevel(logging.DEBUG)

        # 定义handler的输出格式
        formatter = logging.Formatter(log_fmt)
        fh.setFormatter(formatter)

        # 给logger添加handler
        logger.addHandler(fh)

        # 记录日志
        level_dic = {'debug': logger.debug, 'info': logger.info, 'warning': logger.warning, 'error': logger.error,
                     'critical': logger.critical}
        level_dic[log_level](log_message)

        # 删除重复记录
        fh.flush()
        logger.removeHandler(fh)


ins_log = Logger()


def timeit(func):
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        duration = end_time - start_time
        ins_log.read_log('info', '%s execute duration :%.3f second' % (str(func), duration))
        return result

    return wrapper


# ins_log.write_log('info', 'xxxx')
# ins_log.read_log('info', 'xxxx')



import json
from websdk.tools import singleton
from .apis import AdminAPIS, TaskAPIS, KerriganAPIS


@singleton
class ConstAPIS(AdminAPIS, TaskAPIS, KerriganAPIS):
    def __init__(self):
        pass

    def __setattr__(self, name, value):
        if name in self.__dict__:
            raise TypeError("Can't rebind const (%s)" % name)

        if not isinstance(value, dict):
            raise TypeError("Value must be in dict format")

        if not value.get('url'):
            raise TypeError("Value must have url")

        if not value.get('description'):
            raise TypeError("Value must have description")

        if value.get('body'):
            if not isinstance(value.get('body'), dict):
                json.loads(value)

        self.__dict__[name] = value


api_set = ConstAPIS()



import redis
from .consts import const
from .configs import configs

cache_conns = {}


def cache_conn(key=None, db=None):
    redis_configs = configs[const.REDIS_CONFIG_ITEM]
    if not key:
        key = const.DEFAULT_RD_KEY
    for config_key, redis_config in redis_configs.items():
        auth = redis_config[const.RD_AUTH_KEY]
        host = redis_config[const.RD_HOST_KEY]
        port = redis_config[const.RD_PORT_KEY]
        password = redis_config[const.RD_PASSWORD_KEY]
        if db:
            db = db
        else:
            db = redis_config[const.RD_DB_KEY]
        return_utf8 = False
        if const.RD_DECODE_RESPONSES in redis_config:
            return_utf8 = redis_config[const.RD_DECODE_RESPONSES]

        if auth:
            redis_pool = redis.ConnectionPool(host=host, port=port, db=db, password=password,
                                              decode_responses=return_utf8)
        else:
            redis_pool = redis.ConnectionPool(host=host, port=port, db=db, decode_responses=return_utf8)
        cache_conns[config_key] = redis.StrictRedis(connection_pool=redis_pool)
    return cache_conns[key]



"""
记录API
"""


class TaskAPIS:
    job_prefix = "/api/job"
    create_jobs = dict(method='POST',
                       url=f'{job_prefix}/v2/flow/accept/create/',
                       body={
                           "order_name": "标题",
                           "creator": "发起人",
                           "flow_version_name": "依赖的流程名称",
                           "start_node": "选择的开始节点",
                           "start_time": "2018-11-27 14:09:50",
                           "global_params": "{}",  # 参数，必须为json格式且可以格式化为字典
                           "global_node": "{}",  # 初始的执行节点
                           "details": "这里是备注",
                           "classify": "",  # 类型
                           "hosts": "{1: '127.0.0.1'}"
                       },
                       field_help={
                           "order_name": "标题",
                           "creator": "提交人",
                           "flow_version_name": "依赖的流程名称",
                           "start_node": "如果有多个开始节点，必须指定一个",
                           "start_time": "在开始节点上设置时间，到这个时间会进行执行",
                           "global_node": "初始的执行节点",
                           "global_params": "这里是一个字典，里面的参数可以自行定义，如上，你模板参数里面用到了哪些你都可以在这里定义出来，当你的POST到这个接口时候，我们会自动接受此参数，并帮你运行脚本 解析你要传入的参数。",
                           "details": "描述，备注信息",
                       },
                       description='基于此接口可以创建作业任务')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2018/12/11
Desc    : 
"""

import os
import time
import json
import socket
import smtplib
from .consts import const
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from aliyunsdkdysmsapi.request.v20170525 import SendSmsRequest, QuerySendDetailsRequest
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.profile import region_provider
import uuid


class SendMail:
    def __init__(self, mail_host, mail_port, mail_user, mail_password, mail_ssl=False, mail_tls=False):
        """
        :param mail_host:     SMTP主机
        :param mail_port:     SMTP端口
        :param mail_user:     SMTP账号
        :param mail_password: SMTP密码
        :param mail_ssl:      SSL=True, 如果SMTP端口是465，通常需要启用SSL，  如果SMTP端口是587，通常需要启用TLS
        """
        self.mail_host = mail_host
        self.mail_port = mail_port
        self.__mail_user = mail_user
        self.__mail_password = mail_password
        self.mail_ssl = mail_ssl
        self.mail_tls = mail_tls

    def send_mail(self, to_list, subject, content, subtype='plain', att=None):
        """
        :param to_list:  收件人，多收件人半角逗号分割， 必填
        :param subject:  标题， 必填
        :param content:  内容， 必填
        :param subtype:  格式，默认：plain, 可选html
        :param att:      附件，支持单附件，选填
        """
        msg = MIMEMultipart()
        msg['Subject'] = subject  # 标题
        msg['From'] = self.__mail_user  # 发件人
        msg['To'] = to_list  # 收件人，必须是一个字符串
        # 邮件正文内容
        msg.attach(MIMEText(content, subtype, 'utf-8'))
        if att:
            if not os.path.isfile(att):
                raise FileNotFoundError('{0} file does not exist'.format(att))

            dirname, filename = os.path.split(att)
            # 构造附件1，传送当前目录下的 test.txt 文件
            att1 = MIMEText(open(att, 'rb').read(), 'base64', 'utf-8')
            att1["Content-Type"] = 'application/octet-stream'
            # 这里的filename可以任意写，写什么名字，邮件中显示什么名字
            att1["Content-Disposition"] = 'attachment; filename="{0}"'.format(filename)
            msg.attach(att1)

        try:
            if self.mail_ssl:
                '''SSL加密方式，通信过程加密，邮件数据安全, 使用端口465'''
                # print('Use SSL SendMail')
                server = smtplib.SMTP_SSL()
                server.connect(self.mail_host, self.mail_port)  # 连接服务器
                server.login(self.__mail_user, self.__mail_password)  # 登录操作
                server.sendmail(self.__mail_user, to_list.split(','), msg.as_string())
                server.close()
            elif self.mail_tls:
                # print('Use TLS SendMail')
                '''使用TLS模式'''
                server = smtplib.SMTP()
                server.connect(self.mail_host, self.mail_port)  # 连接服务器
                server.starttls()
                server.login(self.__mail_user, self.__mail_password)  # 登录操作
                server.sendmail(self.__mail_user, to_list.split(','), msg.as_string())
                server.close()
                return True
            else:
                '''使用普通模式'''
                server = smtplib.SMTP()
                server.connect(self.mail_host, self.mail_port)  # 连接服务器
                server.login(self.__mail_user, self.__mail_password)  # 登录操作
                server.sendmail(self.__mail_user, to_list.split(','), msg.as_string())
                server.close()
                return True
        except Exception as e:
            print(str(e))
            return False


class SendSms(object):
    def __init__(self, REGION, DOMAIN, PRODUCT_NAME, sms_access_key_id, sms_access_key_secret, ):
        self.acs_client = AcsClient(sms_access_key_id, sms_access_key_secret, REGION)
        region_provider.add_endpoint(PRODUCT_NAME, REGION, DOMAIN)

    def send_sms(self, phone_numbers, template_param=None, sign_name="ops", template_code=""):
        business_id = uuid.uuid1()
        sms_request = SendSmsRequest.SendSmsRequest()
        # 申请的短信模板编码,必填
        sms_request.set_TemplateCode(template_code)

        # 短信模板变量参数
        if template_param is not None:
            sms_request.set_TemplateParam(template_param)

        # 设置业务请求流水号，必填。
        sms_request.set_OutId(business_id)

        # 短信签名
        sms_request.set_SignName(sign_name)

        # 短信发送的号码列表，必填。
        sms_request.set_PhoneNumbers(phone_numbers)

        # 调用短信发送接口，返回json
        sms_response = self.acs_client.do_action_with_exception(sms_request)

        # 业务处理
        return sms_response

    def query_send_detail(self, biz_id, phone_number, page_size, current_page, send_date):
        query_request = QuerySendDetailsRequest.QuerySendDetailsRequest()
        # 查询的手机号码
        query_request.set_PhoneNumber(phone_number)
        # 可选 - 流水号
        query_request.set_BizId(biz_id)
        # 必填 - 发送日期 支持30天内记录查询，格式yyyyMMdd
        query_request.set_SendDate(send_date)
        # 必填-当前页码从1开始计数
        query_request.set_CurrentPage(current_page)
        # 必填-页大小
        query_request.set_PageSize(page_size)

        # 数据提交方式
        # queryRequest.set_method(MT.POST)

        # 数据提交格式
        # queryRequest.set_accept_format(FT.JSON)

        # 调用短信记录查询接口，返回json
        query_response = self.acs_client.do_action_with_exception(query_request)
        # print(query_response.decode('utf-8'))

        return query_response


def mail_login(user, password, mail_server='smtp.exmail.qq.com'):
    # 模拟登录来验证邮箱
    try:
        server = smtplib.SMTP()
        server.connect(mail_server)
        server.login(user, password)
        return True
    except Exception as e:
        print(user, e)
        return False


def get_contain_dict(src_data: dict, dst_data: dict) -> bool:
    if not isinstance(src_data, dict):
        try:
            src_data = json.loads(src_data)
        except Exception as err:
            return False

    if not isinstance(dst_data, dict):
        try:
            dst_data = json.loads(dst_data)
        except Exception as err:
            return False

    # src_key = list(src_data.keys())
    # dst_key = list(dst_data.keys())
    pd = [False for c in src_data.keys() if c not in dst_data]
    if pd:
        return False
    else:
        src_val = list(src_data.values())
        dst_val = list(dst_data.values())
        pds = [False for c in src_val if c not in dst_val]
        if pds:
            return False
        else:
            return True


def now_time_stamp() -> int:
    """
    秒时间戳
    :return: int
    """
    return int(time.time())


# 这个地址具有唯一性
def get_node_address():
    node_name = os.getenv(const.NODE_ADDRESS) if os.getenv(const.NODE_ADDRESS) else socket.gethostname()
    mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
    return f'{node_name}--mac-{mac}'


# 这个地址是默认可以通配的
def get_node_topic(node=False):
    if not node:
        if os.getenv(const.NODE_ADDRESS): return os.getenv(const.NODE_ADDRESS) + '#'
        mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
        return f'{socket.gethostname()}--mac-{mac}#'
    else:
        if os.getenv(const.NODE_ADDRESS): return os.getenv(const.NODE_ADDRESS)
        mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
        return f'{socket.gethostname()}--mac-{mac}'


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Version : 0.0.8
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2021/1/26 20:28
Desc    :  https://github.com/tiangolo/pydantic-sqlalchemy
"""

####
from typing import Container, Optional, Type
from pydantic import BaseConfig, BaseModel, create_model, ValidationError
from sqlalchemy.inspection import inspect
from sqlalchemy.orm.properties import ColumnProperty


# 删除的时候一般只有id
class PydanticDel(BaseModel):
    id: int


class OrmConfig(BaseConfig):
    orm_mode = True


def sqlalchemy_to_pydantic(db_model: Type, *, config: Type = OrmConfig, exclude: Container[str] = []) -> Type[
    BaseModel]:
    mapper = inspect(db_model)
    fields = {}
    for attr in mapper.attrs:
        if isinstance(attr, ColumnProperty):
            if attr.columns:
                name = attr.key
                if name in exclude:
                    continue
                column = attr.columns[0]
                python_type: Optional[type] = None
                if hasattr(column.type, "impl"):
                    if hasattr(column.type.impl, "python_type"):
                        python_type = column.type.impl.python_type
                elif hasattr(column.type, "python_type"):
                    python_type = column.type.python_type
                assert python_type, f"Could not infer python_type for {column}"
                default = None
                if column.default is None and not column.nullable:
                    default = ...
                fields[name] = (python_type, default)
    pydantic_model = create_model(
        db_model.__name__, __config__=config, **fields  # type: ignore
    )
    return pydantic_model



"""
mysql操作
"""
import pymysql
from opssdk.logs import Log


class MysqlBase:
    def __init__(self, **args):
        self.host = args.get('host')
        self.user = args.get('user')
        self.pswd = args.get('passwd')
        self.db = args.get('db')
        self.port = int(args.get('port', 3306))
        self.charset = args.get('charset', 'utf8')

        log_path = '/log/yunwei/yunwei_mysql.log'
        self.log_ins = Log('111', log_path)
        try:
            self.conn = pymysql.connect(host=self.host, user=self.user,
                                        password=self.pswd, db=self.db, port=self.port, charset=self.charset)
            self.cur = self.conn.cursor()
        except:
            raise ValueError('mysql connect error {0}'.format(self.host))

    # 释放资源
    def __del__(self):
        self.cur.close()
        self.conn.close()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cur.close()
        self.conn.close()

    # 查询
    def query(self, sql):
        try:
            self.cur.execute(sql)  # 执行sql语句
            res = self.cur.fetchall()  # 获取查询的所有记录
        except Exception as e:
            self.log_ins.write_log("error", e)
            raise e

        return res

    def change(self, sql):
        resnum = 0
        try:
            resnum = self.cur.execute(sql)
            # 提交
            self.conn.commit()
        except Exception as e:
            # 错误回滚
            self.log_ins.write_log("error", e)
            self.conn.rollback()
        return resnum



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2018年2月5日13:37:54
Desc    ：记录API
"""


class AdminAPIS:
    route_prefix = "/api/mg"
    get_users = dict(method='GET',
                     url=f'{route_prefix}/v3/accounts/user/',
                     params={
                         'page': 1,
                         'limit': 201
                     },
                     field_help={},
                     description='获取用户信息')

    opt_users = dict(method='POST',
                     url=f'{route_prefix}/v3/accounts/user/',
                     body={
                         'username': None,
                         'nickname': None,
                         'password': None,
                         'department': None,
                         'tel': None,
                         'wechat': None,
                         'no': None,
                         'email': None,
                         'user_state': '20',
                     },
                     field_help={
                         'user_state': '20',
                     },
                     description='操作用户数据，支持增删改，请修改method和body数据')

    get_resource_info_by_user = dict(method='GET',
                                     url='/mg/v2/overall/resource/user/',
                                     params={
                                         'nickname': None,  ## 默认为当前用户有权限的
                                         'expand': None  ## yes 是扩展为目录的 ， no 是可以当全局标签的
                                     },
                                     field_help={
                                         'nickname': None,  ## 默认为当前用户有权限的
                                         'expand': None  ## yes 是扩展为目录的 ， no 是可以当全局标签的，没有则是全部
                                     },
                                     description='获取用户有权限的资源组/目录')

    get_resource_info = dict(method='GET',
                             url='/mg/v2/overall/resource/',
                             params={
                                 'key': None,  ## 筛选字段
                                 'value': None  ## 筛选字段值
                             },
                             field_help={
                                 'key': None,  ## 筛选字段
                                 'value': None  ## 筛选字段值
                             },
                             description='获取资源组/目录')

    get_role_list = dict(method='GET',
                         url=f'{route_prefix}/v3/accounts/role/',
                         params={
                         },
                         field_help={
                             'page': '分页/第几页',  ### 分页/第几页
                             'limit': '分页/每页多少个',  ### 分页/每页多少个
                             'value': '模糊查询'  ### 模糊查询
                         },
                         description='获取角色信息')

    get_all_role_user = dict(method='GET',
                             url=f'{route_prefix}/v3/accounts/all_role_user/',
                             params={
                             },
                             field_help={
                                 'page': '分页/第几页',  ### 分页/第几页
                                 'limit': '分页/每页多少个',  ### 分页/每页多少个
                                 'value': '模糊查询'  ### 模糊查询
                             },
                             description='获取用户组和用户组内用户信息')

    get_send_addr = dict(method='GET',
                         url=f'{route_prefix}/v1/notifications/send_addr/',
                         params={
                         },
                         field_help={
                             'users_str': '用户  半角逗号分隔',  ### 用户  半角逗号分隔
                             'notice_group_str': '通知组  半角逗号分隔',  ### 通知组  半角逗号分隔
                             'roles_str': '角色   半角逗号分隔'  ### 角色   半角逗号分隔
                         },
                         description='获取用户的联系方式，手机/邮箱/钉钉ID')

    get_notice_group = dict(method='GET',
                            url=f'{route_prefix}/v3/notifications/group/',
                            params={
                                'page': 1,
                                'limit': 201,
                                'value': ''
                            },
                            field_help={
                                'page': '分页/第几页',  # 分页/第几页
                                'limit': '分页/每页多少个',  # 分页/每页多少个
                                'value': '模糊查询'  # 模糊查询
                            },
                            description='获取通知组')

    opt_notice_group = dict(method='POST',
                            url=f'{route_prefix}/v3/notifications/group/',
                            body={
                            },
                            field_help={
                            },
                            description='操作通知组')

    send_notice = dict(method='POST',
                       url=f'{route_prefix}/v3/notifications/factory/',
                       body={
                           "name": 'default',
                           "msg": {"msg": "这个即将发布的新版本，创始人xx称它为红树林。而在此之前，每当面临重大升级"}
                       },
                       field_help={
                           "name": '模板名称',
                           "msg": "发送的消息变量字典",
                           "notice_conf": "随着通知一起传入的配置信息，会覆盖模板里面的配置信息"
                       },
                       description='根据通知模板发送通知')

    send_update_notice = dict(method='PUT',
                              url=f'{route_prefix}/v3/notifications/factory/',
                              body={
                                  "agent_id": 27689,
                                  "status_value": "已同意",
                                  "status_bg": "0xFF78C06E",
                                  "task_id": 37491848
                              },
                              field_help={
                                  "agent_id": '申请的应用id',
                                  "status_value": "状态值",
                                  "status_bg": "颜色",
                                  "task_id": "已经发送消息ID，发送通知的时候可以获取他的返回"
                              },
                              description='变更已发出通知消息的状态， 目前只有钉钉工作通知OA方式可用')

    send_custom_notice = dict(method='POST',
                              url=f'{route_prefix}/v1/notifications/custom/',
                              body={
                                  "send_addr": '',
                                  "userid_list": '',
                                  "msg": {"msg": "这个即将发布的新版本，创始人xx称它为红树林。而在此之前，每当面临重大升级"}
                              },
                              field_help={
                                  "send_addr": '需要的通知的用户信息',
                                  "userid_list": "",
                                  "msg": "发送的消息变量字典",
                              },
                              description='自定义的通知信息')



"""
处理API请求
"""

import json
import requests
from urllib.parse import urlencode
import logging
from tornado.httpclient import AsyncHTTPClient

logger = logging.getLogger(__name__)


class AcsClient:
    def __init__(self, request=None, auth_key=None, headers=None, endpoint='http://gw.opendevops.cn',
                 request_timeout=10):
        if request:
            self.headers = request.headers
        elif headers:
            self.headers = headers
        else:
            self.headers = {"Cookie": 'auth_key={}'.format(auth_key)}

        if 'If-None-Match' in self.headers: del self.headers['If-None-Match']
        self.endpoint = endpoint
        self.headers['Sdk-Method'] = 'zQtY4sw7sqYspVLrqV'
        self.request_timeout = request_timeout

    # 设置返回为json
    def do_action(self, **kwargs):
        kwargs = self.with_params_data_url(**kwargs)
        response = requests.request(kwargs.get('method'), kwargs.get('url'), headers=self.headers,
                                    data=kwargs.get('body'), timeout=self.request_timeout)

        return response.text

    # 返回完整信息
    def do_action_v2(self, **kwargs):
        kwargs = self.with_params_data_url(**kwargs)
        response = requests.request(kwargs.get('method'), kwargs.get('url'), headers=self.headers,
                                    data=kwargs.get('body'), timeout=self.request_timeout)
        return response

    async def do_action_with_async(self, **kwargs):

        body = await self._implementation_of_do_action(**kwargs)
        return body

    async def _implementation_of_do_action(self, **kwargs):
        http_client = AsyncHTTPClient()
        request = self.with_params_data_url(**kwargs)

        response = await http_client.fetch(request.get('url'), method=request.get('method'), raise_error=False,
                                           body=request.get('body'), headers=self.headers,
                                           request_timeout=self.request_timeout)

        return response.body

    def with_params_data_url(self, **kwargs):
        # 重新组装URL
        url = "{}{}".format(self.endpoint, kwargs['url'])
        kwargs['url'] = url

        if not kwargs['method']: kwargs['method'] = 'GET'

        body = kwargs.get('body', {})

        if kwargs['method'] in ['POST', 'post', 'PATCH', 'patch', 'PUT', 'put']:
            if not body:
                raise TypeError('method {},  body can not be empty'.format(kwargs['method']))
            else:
                if not isinstance(body, dict):  json.loads(body)

        if body and isinstance(body, dict): kwargs['body'] = json.dumps(body)

        params = kwargs.get('params')
        if params: kwargs['url'] = "{}?{}".format(url, urlencode(params))

        if not self.headers: self.headers = kwargs.get('headers', {})

        if kwargs['method'] not in ['GET', 'get']: self.headers['Content-Type'] = 'application/json'

        return kwargs

    @staticmethod
    def help():
        help_info = """
        headers = {"Cookie": 'auth_key={}'.format(auth_key)}
        ### 三种实例化方式
        1. client = AcsClient(endpoint=endpoint, headers=headers)
        2. client = AcsClient(endpoint=endpoint, request=self.request)
        3. client = AcsClient(endpoint=endpoint, auth_key=auth_key)
        
        调用： 传入api 的参数，可以参考下面示例
        
        同步
        response = client.do_action(**api_set.get_users) 
        print(json.loads(response))
        
        异步
        # import asyncio
        # loop = asyncio.get_event_loop()
        # ### 使用gather或者wait可以同时注册多个任务，实现并发
        # # task1 = asyncio.ensure_future(coroutine1)
        # # task2 = asyncio.ensure_future(coroutine2)
        # # tasks = asyncio.gather(*[task1, task2])
        # # loop.run_until_complete(tasks)
        # ### 单个使用
        # response = loop.run_until_complete(client.do_action_with_async(**api_set.get_users))
        # response = json.loads(response)
        # print(response)
        # loop.close()
        
        tornado 项目内必须使用异步，不过可以直接使用
        from websdk.client import AcsClient
        from websdk.api_set import api_set
        async def get(self):
            endpoint = ''
            client = AcsClient(endpoint=endpoint, headers=self.request.headers)
            response = await client.do_action_with_async(**api_set.get_users)
            return self.write(response)
        
         """
        return help_info


if __name__ == '__main__':
    pass



"""
TREELAB OPS SDK
"""

import sys
from distutils.core import setup

version = '0.0.1'

if sys.version_info < (2, 7) or (3, 0) <= sys.version_info < (3, 9):
    print('Finit requires at least Python 2.7 or 3.9 to run.')
    sys.exit(1)


def get_data_files():
    data_files = [
        ('share/doc/ops_sdk', ['README.md'])
    ]
    return data_files


def get_install_requires():
    requires = ['fire==0.3.1', 'shortuuid==1.0.1', 'pymysql==1.0.2', 'sqlalchemy==1.3.23', 'pika==1.2.0',
                'PyJWT==2.0.1', 'Crypto==1.4.1', 'requests==2.25.1', 'redis==4.3.4', 'tornado>=6.0',
                'cryptography==3.4.8', 'aliyun-python-sdk-core-v3==2.13.11', 'aliyun-python-sdk-dysmsapi==2.1.1',
                'python-dateutil==2.7.5', 'ldap3==2.6', 'pydantic==1.7', 'pycryptodome==3.9.9', 'rsa==4.0']
    return requires


setup(
    name='treelabOpssdk',
    version=version,
    description="treelab运维SDK",
    packages=['opssdk', 'opssdk.logs', 'opssdk.operate', 'opssdk.install', 'opssdk.get_info', 'opssdk.utils', 'websdk', 'websdk.apis', 'websdk.cloud', 'websdk.utils'],
    license='GPLv3',
    keywords="ops,opencodo,devops",
    install_requires=get_install_requires(),
    long_description='SDK of the operation and maintenance script logs operate',
    include_package_data=True,
    data_files=get_data_files(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console :: Curses',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    platforms='any'
)
